#ifndef RENDERINGMAX_H_INCLUDED
#define RENDERINGMAX_H_INCLUDED

#include <mysql++.h>
#include <ssqls.h>
#include <vector>
#include <string>

using namespace std;

// The following is calling a very complex macro which will create
// "struct stock", which has the member variables:
//
//   sql_char item;
//   ...
//   Null<sql_mediumtext> description;
//
// plus methods to help populate the class from a MySQL row.  See the
// SSQLS sections in the user manual for further details.
namespace CloudResourceManager
{
    struct Users;
    struct Jobs;
    struct JobStatus;
    struct JobTypes;
    struct NotificationTypes;
    struct Tasks;
    struct TaskStatus;
    struct TaskTypes;
    struct UserCredits;
    struct EC2_Instances;
    struct EC2_InstanceStatus;
    struct EC2_InstanceTypes;
    struct EC2_Requests;
    struct EC2_RequestStatus;
    struct EC2_RequestTypes;
    struct UserCredits_View;
    struct Tasks_View;
    struct Jobs_View;
    struct EC2_Instances_View;
    struct EC2_Requests_View;

    namespace JobTypeEnum
    {
        enum
        {
            S3Upload=1,
            S3Download = 2,
            BlenderRender=3
        };
    };

    namespace EC2InstanceStatusEnum
    {
        enum
        {
            New=1,
            Open=2,
            Active=3,
            Initiate=4,
            Ready=5,
            Idle=6,
            Failed=7,
            Closed=8,
            Error=10
        };
    }

    namespace JobStatusEnum
    {
        enum
        {
            New=1,
            Initiate=2,
            Start=3,
            Running=4,
            Finish=5,
            Abort=6,
            Suspend=7,
            Idle = 8,
            Unknown=9,
            Error=10
        };
    }

    namespace TaskStatusEnum
    {
        enum
        {
            New=1,
            Initiate=2,
            Start=3,
            Running=4,
            Finish=5,
            Abort=6,
            Suspend=7,
            Error=10
        };
    }

    namespace TaskTypeEnum
    {
        enum
        {
            Blender_Rendering=1
        };
    }

    namespace NotificationTypeEnum
    {
        enum
        {
            NoNotification = 0,
            Email =1,
            Text = 2
        };
    }

    namespace EC2RequestStatusEnum
    {
        enum
        {
           New=1,
           Open=2,
           Active=3,
           Failed=4,
           Closed= 5,
           Canceled =6,
           Error=10
        };
    }

    class MySqlppHelper
    {
    private:
        mysqlpp::Connection con;
        static std::string db;
        static std::string server;
        static std::string user;
        static std::string pass;
        static bool bInitialized;
        static int port;
    public:
        MySqlppHelper();
        bool GetUsersById(int id, Users& user);
        void ShowUsers(const Users& user);
        void ShowTasks(const Tasks& task);
        void ShowJobs(const Jobs& job);
        void ShowEC2_Instances(const EC2_Instances& instance);
        void ShowEC2_Requests(const EC2_Requests& request);
        void ShowUserCredits(const UserCredits& credit);
        bool GetTasksById(int id, Tasks& task);
        bool GetJobsById(int id, Jobs& job);
        //bool GetJobStatusById(int id, JobStatus& jobStatus);
        //bool GetJobTypesById(int id, JobTypes& jobType);
        //bool GetNotificationTypesById(int id, NotificationTypes& notificationType);
       // bool GetTaskStatusById(int id, TaskStatus& taskStatus);
        //bool GetTaskTypesById(int id, TaskTypes& taskType);
        bool GetUserCreditsById(int id, UserCredits& credit);
        bool GetEC2_InstancesById(int id, EC2_Instances& instance);
        //bool GetEC2_InstanceStatusById(int id, EC2_InstanceStatus& status);
        bool GetEC2_InstanceTypesById(int id, EC2_InstanceTypes& type);
        bool GetEC2_RequestsById(int id, EC2_Requests& request);
        //bool GetEC2_RequestStatusById(int id, EC2_RequestStatus& status);
        //bool GetEC2_RequestTypesById(int id, EC2_RequestTypes& type);
        bool GetUserCredits_ViewById(int id, UserCredits_View& credit);
        bool GetTasks_ViewById(int id, Tasks_View& task);
        bool GetJobs_ViewById(int id, Jobs_View& job);
        bool GetEC2_Instances_ViewById(int id, EC2_Instances_View& instance);
        bool GetEC2_Requests_ViewById(int id, EC2_Requests_View& request);
        void ShowUserCredits_View(const UserCredits_View& credit);


        bool UpdateJobStatusId(int jobId, int statusId);
        vector<Tasks> GetNewTasks();
        vector<EC2_Requests> GetNewEC2Requests();
        vector<EC2_Requests> GetOpenEC2Requests();
        vector<EC2_Requests> GetActiveEC2Requests();
        vector<EC2_Instances> GetNewEC2Instances();
        vector<EC2_Instances> GetOpenEC2Instances();
        vector<EC2_Instances> GetActiveEC2Instances();
        vector<EC2_Instances> GetInitiateEC2Instances();
        vector<Tasks> GetStartTasks();
        vector<Jobs> GetIncompleteJobsByTaskId(int taskId);

        vector<EC2_Instances> GetEC2InstancesByUserId(int userId);

        vector<EC2_Instances> GetFinishedEC2Instances();

        vector<Jobs_View> GetInitiateJobs(int tasksId);
        int SaveNewJobs(const Jobs& job);
        int SaveNewTasks(const Tasks& task);
        int SaveNewEC2_Instances(const EC2_Instances& instance);
        int SaveNewEC2_Requests(const EC2_Requests& request);
        int SaveNewUserCredits(const UserCredits& userCredit);
        int SaveNewUsers(const Users& user);
        bool UpdateUsers(int userId, const Users& user);
        bool UpdateJobs(int jobId, const Jobs& job);
        bool UpdateTasks(int taskId, const Tasks& task);
        bool UpdateEC2_Instances(int instanceId, const EC2_Instances& instance);
        bool UpdateEC2_Requests(int requestId, const EC2_Requests& request);
        bool UpdateUserCredits(int creditId, const UserCredits& credit);

        /*
        void showTasks_View(Tasks_View& task);
        void showJobs_View(Jobs_View& job);
        void showEC2_Instances_View(EC2_Instances_View& instance);
        void showRequests_View(Requests_View& request);
        */
    };

    sql_create_6(Users,
        1, 6,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_datetime, LastActivityDate,
        mysqlpp::sql_bigint_unsigned, Credits,
        mysqlpp::sql_blob, password,
        mysqlpp::sql_char, email)


    sql_create_21(Jobs,
        1, 6,
        mysqlpp::sql_bigint_unsigned, JobId,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_bigint_unsigned, JobStatusId,
        mysqlpp::sql_char, CondorJobId,
        mysqlpp::sql_bigint_unsigned, JobTypeId,
        mysqlpp::sql_char, JobOutPutFolder,
        mysqlpp::sql_char, JobInputFolder,
        mysqlpp::sql_char, JobError,
        mysqlpp::sql_char, JobInputMainFile,
        mysqlpp::sql_char, JobOutputMainFile,
        mysqlpp::sql_char, JobDesc,
        mysqlpp::sql_char, JobComment,
        mysqlpp::sql_datetime, JobStartTime,
        mysqlpp::sql_datetime, JobEndTime,
        mysqlpp::sql_datetime, JobSubmitTime,
        mysqlpp::sql_bigint_unsigned, JobRunTime,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_char, JobInputParameter)


    sql_create_2(JobStatus,
        1, 2,
        mysqlpp::sql_bigint_unsigned, StatusId,
        mysqlpp::sql_char, StatusName)


    sql_create_2(JobTypes,
        1, 2,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_char, TypeName)

    sql_create_2(NotificationTypes,
        1, 2,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_char, TypeName)

    sql_create_20(Tasks,
        1, 20,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_bigint_unsigned, TaskTypeId,
        mysqlpp::sql_bigint_unsigned, TaskStatusId,
        mysqlpp::sql_char, TaskInputParameter,
        mysqlpp::sql_char, TaskInputFolder,
        mysqlpp::sql_char, TaskOutputFolder,
        mysqlpp::sql_char, TaskMainFile,
        mysqlpp::sql_bigint_unsigned, MaxInstanceNumber,
        mysqlpp::sql_bool, SpotInstance,
        mysqlpp::sql_bigint_unsigned, EC2InstanceTypeId,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_char, EC2SnapshotId,
        mysqlpp::sql_char, TaskDesc,
        mysqlpp::sql_char, TaskComment,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_bigint_unsigned, ModifiedBy,
        mysqlpp::sql_bigint_unsigned, NotificationTypeId
        )

    sql_create_2(TaskStatus,
        1, 2,
        mysqlpp::sql_bigint_unsigned, StatusId,
        mysqlpp::sql_char, StatusName)

    sql_create_2(TaskTypes,
        1, 2,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_char, TypeName)


    sql_create_8(UserCredits,
        1, 8,
        mysqlpp::sql_bigint_unsigned, CreditId,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_bigint_unsigned, Value,
        mysqlpp::sql_bigint_unsigned, CurrencyId,
        mysqlpp::sql_double, CurrencyAmount,
        mysqlpp::sql_bigint_unsigned, VerifiedBy,
        mysqlpp::sql_bool, Verified,
        mysqlpp::sql_datetime, VerifiedOn)

    sql_create_19(EC2_Instances,
        1, 19,
        mysqlpp::sql_bigint_unsigned, InstanceId,
        mysqlpp::sql_bigint_unsigned, StatusId,
        mysqlpp::sql_bigint_unsigned, ClusterId,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_datetime, InstanceStartTime,
        mysqlpp::sql_datetime, InstanceEndTime,
        mysqlpp::sql_char, InstanceDesc,
        mysqlpp::sql_char, InstanceComment,
        mysqlpp::sql_char, EC2_InstanceId,
        mysqlpp::sql_char, IP,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_double, InstancePrice,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_char, PublicDNS,
        mysqlpp::sql_char, PrivateDNS,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_bigint_unsigned, ModifiedBy)

    sql_create_2(EC2_InstanceStatus,
        1, 2,
        mysqlpp::sql_bigint_unsigned, StatusId,
        mysqlpp::sql_char, StatusName)

    sql_create_4(EC2_InstanceTypes,
        1, 4,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_char, EC2TypeName,
        mysqlpp::sql_char, PlatformName,
        mysqlpp::sql_char, EC2AMIId)

    sql_create_15(EC2_Requests,
        1, 15,
        mysqlpp::sql_bigint_unsigned, RequestId,
        mysqlpp::sql_bigint_unsigned, RequestTypeId,
        mysqlpp::sql_bigint_unsigned, RequestStatusId,
        mysqlpp::sql_bigint_unsigned, InstanceId,
        mysqlpp::sql_datetime, RequestStartTime,
        mysqlpp::sql_datetime, RequestEndTime,
        mysqlpp::sql_char, EC2_RequestId,
        mysqlpp::sql_char, RequestDesc,
        mysqlpp::sql_char, RequestComment,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_bigint_unsigned, ModifiedBy)

    sql_create_2(EC2_RequestStatus,
        1, 2,
        mysqlpp::sql_bigint_unsigned, StatusId,
        mysqlpp::sql_char, StatusName)

    sql_create_2(EC2_RequestTypes,
        1, 2,
        mysqlpp::sql_bigint_unsigned, TypeId,
        mysqlpp::sql_char, TypeName)

    sql_create_12(UserCredits_View,
        1, 12,
        mysqlpp::sql_bigint_unsigned, CreditId,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_bigint_unsigned, Value,
        mysqlpp::sql_bigint_unsigned, CurrencyId,
        mysqlpp::sql_char, CurrencyName,
        mysqlpp::sql_double, CurrencyAmount,
        mysqlpp::sql_bigint_unsigned, VerifiedBy,
        mysqlpp::sql_bool, Verified,
        mysqlpp::sql_datetime, VerifiedOn,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_datetime, LastActivityDate,
        mysqlpp::sql_bigint_unsigned, Credits)

    sql_create_23(Tasks_View,
        1, 23,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_bigint_unsigned, Credits,
        mysqlpp::sql_bigint_unsigned, TaskTypeId,
        mysqlpp::sql_bigint_unsigned, TaskStatusId,
        mysqlpp::sql_char, TaskStatusName,
        mysqlpp::sql_char, TaskInputParameter,
        mysqlpp::sql_char, TaskInputFolder,
        mysqlpp::sql_char, TaskOutputFolder,
        mysqlpp::sql_char, TaskMainFile,
        mysqlpp::sql_bigint_unsigned, MaxInstanceNumber,
        mysqlpp::sql_bool, SpotInstance,
        mysqlpp::sql_bigint_unsigned, EC2InstanceTypeId,
        mysqlpp::sql_char, EC2TypeName,
        mysqlpp::sql_char, PlatformName,
        mysqlpp::sql_char, EC2AMIId,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_bigint_unsigned, ModifiedBy,
        mysqlpp::sql_bigint_unsigned, NotificationTypeId,
        mysqlpp::sql_char, NotificationName)

    sql_create_34(Jobs_View,
        1, 34,
        mysqlpp::sql_bigint_unsigned, JobId,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_bigint_unsigned, JobStatusId,
        mysqlpp::sql_char, JobStatusName,
        mysqlpp::sql_char, CondorJobId,
        mysqlpp::sql_bigint_unsigned, JobTypeId,
        mysqlpp::sql_char, JobTypeName,
        mysqlpp::sql_char, JobOutPutFolder,
        mysqlpp::sql_char, JobInputFolder,
        mysqlpp::sql_char, JobInputMainFile,
        mysqlpp::sql_char, JobOutputMainFile,
        mysqlpp::sql_datetime, JobStartTime,
        mysqlpp::sql_datetime, JobEndTime,
        mysqlpp::sql_datetime, JobSubmitTime,
        mysqlpp::sql_datetime, JobRunTime,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_char, JobInputParameter,
        mysqlpp::sql_bigint_unsigned, UserId,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_bigint_unsigned, TaskTypeId,
        mysqlpp::sql_bigint_unsigned, TaskStatusId,
        mysqlpp::sql_char, TaskStatusName,
        mysqlpp::sql_char, TaskInputParameter,
        mysqlpp::sql_char, TaskInputFolder,
        mysqlpp::sql_char, TaskOutputFolder,
        mysqlpp::sql_char, TaskMainFile,
        mysqlpp::sql_bigint_unsigned, MaxInstanceNumber,
        mysqlpp::sql_bool, SpotInstance,
        mysqlpp::sql_bigint_unsigned, EC2InstanceTypeId,
        mysqlpp::sql_char, EC2TypeName,
        mysqlpp::sql_char, PlatformName,
        mysqlpp::sql_char, EC2AMIId,
        mysqlpp::sql_bigint_unsigned, NotificationTypeId,
        mysqlpp::sql_char, NotificationName)



    sql_create_32(EC2_Instances_View,
        1, 32,
        mysqlpp::sql_bigint_unsigned, InstanceId,
        mysqlpp::sql_bigint_unsigned, EC2_StatusId,
        mysqlpp::sql_char,EC2_InstanceStatusName,
        mysqlpp::sql_bigint_unsigned, ClusterId,
        mysqlpp::sql_bigint_unsigned, EC2_InstanceTypeId,
        mysqlpp::sql_char,EC2TypeName,
        mysqlpp::sql_datetime, InstanceStartTime,
        mysqlpp::sql_datetime, InstanceEndTime,
        mysqlpp::sql_char, EC2_InstanceId,
        mysqlpp::sql_char, IP,
        mysqlpp::sql_datetime, LastCheckTime,
        mysqlpp::sql_double, InstancePrice,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_bigint_unsigned, Credits,
        mysqlpp::sql_bigint_unsigned, TaskTypeId,
        mysqlpp::sql_bigint_unsigned, TaskStatusId,
        mysqlpp::sql_char, TaskStatusName,
        mysqlpp::sql_char, TaskInputParameter,
        mysqlpp::sql_char, TaskInputFolder,
        mysqlpp::sql_char, TaskOutputFolder,
        mysqlpp::sql_char, TaskMainFile,
        mysqlpp::sql_bigint_unsigned, MaxInstanceNumber,
        mysqlpp::sql_bool, SpotInstance,
        mysqlpp::sql_char, PlatformName,
        mysqlpp::sql_char, EC2AMIId,
        mysqlpp::sql_char, PublicDNS,
        mysqlpp::sql_char, PrivateDNS,
        mysqlpp::sql_datetime, CreatedOn,
        mysqlpp::sql_bigint_unsigned, CreatedBy,
        mysqlpp::sql_datetime, ModifiedOn,
        mysqlpp::sql_bigint_unsigned, ModifiedBy)

    sql_create_29(EC2_Requests_View,
        1, 29,
        mysqlpp::sql_bigint_unsigned, RequestId,
        mysqlpp::sql_bigint_unsigned, RequestTypeId,
        mysqlpp::sql_char, RequestTypeName,
        mysqlpp::sql_bigint_unsigned, RequestStatusId,
        mysqlpp::sql_char, RequstStatusName,
        mysqlpp::sql_bigint_unsigned, InstanceId,
        mysqlpp::sql_bigint_unsigned, EC2_StatusId,
        mysqlpp::sql_char,EC2_InstanceStatusName,
        mysqlpp::sql_bigint_unsigned, ClusterId,
        mysqlpp::sql_bigint_unsigned, EC2_InstanceTypeId,
        mysqlpp::sql_char, EC2TypeName,
        mysqlpp::sql_datetime, InstanceStartTime,
        mysqlpp::sql_datetime, InstanceEndTime,
        mysqlpp::sql_char, EC2_InstanceId,
        mysqlpp::sql_char, IP,
        mysqlpp::sql_bigint_unsigned, TaskId,
        mysqlpp::sql_char, UserName,
        mysqlpp::sql_bigint_unsigned, Credits,
        mysqlpp::sql_bigint_unsigned, TaskTypeId,
        mysqlpp::sql_bigint_unsigned, TaskStatusId,
        mysqlpp::sql_bigint_unsigned, MaxInstanceNumber,
        mysqlpp::sql_bool, SpotInstance,
        mysqlpp::sql_char, PlatformName,
        mysqlpp::sql_char, EC2AMIId,
        mysqlpp::sql_char, PublicDNS,
        mysqlpp::sql_char, PrivateDNS,
        mysqlpp::sql_datetime, RequestStartTime,
        mysqlpp::sql_datetime, RequestEndTime,
        mysqlpp::sql_datetime, LastCheckTime)
}


#endif // RENDERINGMAX_H_INCLUDED
