
#include "EmailManager.h"
#include "CSmtp.h"

using namespace CloudResourceManager;
using namespace std;

bool EmailManager::SendMsg(const MsgParam& msgParam, const MailUserParam& mailUserParam, const MailServerParam& serverParam)
{
    try
    {
        CSmtp mail;
        int i;
        mail.SetSMTPServer(serverParam.smtpServer.c_str(), serverParam.port);

        mail.SetSecurityType((CSmtp::security_type)serverParam.securityType);

        mail.SetLogin(mailUserParam.loginUser.c_str());
        mail.SetPassword(mailUserParam.password.c_str());
        mail.SetSenderName(mailUserParam.senderName.c_str());
        mail.SetSenderMail(mailUserParam.senderEmail.c_str());
        mail.SetReplyTo(mailUserParam.replyEmail.c_str());
        mail.SetSubject(msgParam.subject.c_str());
        for (i = 0; i < msgParam.recipients.size(); i ++)
        {
            mail.AddRecipient(msgParam.recipients[i].c_str());
        }
        for (i = 0; i < msgParam.ccRecipients.size(); i ++)
        {
            mail.AddCCRecipient(msgParam.ccRecipients[i].c_str());
        }
        for (i = 0; i < msgParam.bccRecipients.size(); i ++)
        {
            mail.AddBCCRecipient(msgParam.bccRecipients[i].c_str());
        }

        mail.SetXPriority(XPRIORITY_NORMAL);
        mail.SetXMailer("Modified by Nick");
        mail.AddMsgLine(msgParam.msg.c_str());

        for (i = 0; i < msgParam.attachments.size(); i ++)
        {
            mail.AddAttachment(msgParam.attachments[i].c_str());
        }
        mail.Send();
    }
    catch(ECSmtp e)
	{
		std::cout << "Error: " << e.GetErrorText().c_str() << ".\n";
		return false;
	}

	return true;
}

bool EmailManager::SendHotmailMsg(const MsgParam& msgParam, const MailUserParam& mailUserParam)
{
    MailServerParam server;
    server.smtpServer = "smtp.live.com";
    server.port = 25;
    server.securityType = SECURITY_TYPE::USE_TLS;
    return SendMsg(msgParam, mailUserParam, server);
}

bool EmailManager::SendGmailMsgTLS(const MsgParam& msgParam, const MailUserParam& mailUserParam)
{
    MailServerParam server;
    server.smtpServer = "smtp.gmail.com";
    server.port = 587;
    server.securityType = SECURITY_TYPE::USE_TLS;
    return SendMsg(msgParam, mailUserParam, server);
}

bool EmailManager::SendGmailMsgSSL(const MsgParam& msgParam, const MailUserParam& mailUserParam)
{
    MailServerParam server;
    server.smtpServer = "smtp.gmail.com";
    server.port = 465;
    server.securityType = SECURITY_TYPE::USE_SSL;
    return SendMsg(msgParam, mailUserParam, server);
}

bool EmailManager::SendYahooMsg(const MsgParam& msgParam, const MailUserParam& mailUserParam)
{
    MailServerParam server;
    server.smtpServer = "plus.smtp.mail.yahoo.com";
    server.port = 465;
    server.securityType = SECURITY_TYPE::USE_SSL;
    return SendMsg(msgParam, mailUserParam, server);
}

bool EmailManager::SendReportMail(const string& recipient, const string& subject, const string& msg)
{
    MailUserParam userParam;
    userParam.senderName = "report mail";
    userParam.senderEmail = "nickhuang2009@gmail.com";
    userParam.loginUser = "nickhuang2009";
    userParam.password = "maxrendering";
    userParam.replyEmail = "nickhuang2009@gmail.com";

    MsgParam msgParam;
    msgParam.recipients.push_back(recipient);
    msgParam.subject = subject;
    msgParam.msg = msg;
    return SendGmailMsgSSL(msgParam, userParam);
}
