﻿
#include <string.h>
#include <iostream>
#include <stdexcept>
#include <stdio.h>
#include "log.h"
#include "XmlManager.h"
#include "ConfigureManager.h"
#include "TemplateManager.h"

namespace CloudResourceManager
{
    string TemplateManager::ResolveInternalKey(string strKey, const StringMap& jobContext)
    {
        StringMap::const_iterator it;
        if ((it = jobContext.find(strKey)) != jobContext.end())
        {
            return it->second;
        }
        else
        {
            try
            {
                return ConfigureManager::AppSettings(strKey);
            }
            catch (runtime_error e)
            {
                cout<<e.what()<<endl;
                //ConfigLogger.Instance.LogError(e.Message + "key =" + keyName + " value ="+ keyValue);
                FILE_LOG(logERROR)<<"[ResolveInternalKey]" <<e.what()<<"key =" << strKey;
            }
        }
    }

    // internal utility function, basically rebuild wheels. :)
    int TemplateManager::findStartChar(const char* str, int nStart)
    {
        int index = nStart;
        while (str[index] != '\0')
        {
            if (str[index] == '$')
            {
                break;
            }
            index ++;
        }
        return index;
    }

    int TemplateManager::findEndChar(const char* str, int nStart, char* buffer)
    {
        // skip the '@'
        int index = nStart + 1, i = 0;
        while ((str[index] >='A' && str[index] <='Z')||(str[index] >='0' && str[index] <='9') || (str[index] >='a' && str[index] <='z'))
        {
            buffer[i]= str[index];
            i++;
            index ++;
        }
        buffer[i] = '\0';
        return index;
    }


    int TemplateManager::ReplaceTemplateFile(const string& strTemplateName, const string& strTargetName, const StringMap& hashTable, const StringMap& jobContext)
    {
        int nResult = 0;
        FILE* sr = NULL;
        FILE* sw = NULL;
        if ( (sr = fopen(strTemplateName.c_str(), "r")) != NULL)
        {
            if ((sw = fopen(strTargetName.c_str(), "w")) != NULL)
            {
                try
                {
                    // Read and display lines from the file until the end of
                    // the file is reached.
                    while (!feof(sr))
                    {
                        char line[256];
                        if (!fgets(line, sizeof(line), sr))
                        {
                            break;
                        }
                        //debugging
                        //Console.WriteLine(line);
                        //ConfigLogger.Instance.LogDebug(line);
                        FILE_LOG(logDEBUG)<<"[ReplaceTemplateFile:line]"<<line;
                        int nStart = 0, nEnd = 0, nCurrent = 0, nLength = 0;
                        nStart = findStartChar(line, nStart);
                        while (line[nStart] == '$')
                        {
                            char strKey[256];
                            string strValue;
                            nEnd = findEndChar(line, nStart, strKey);
                            StringMap::const_iterator it = hashTable.find(string(strKey));
                            if (it != hashTable.end())
                            {
                                strValue = it->second;
                            }
                            else
                            {
                                //Console.WriteLine("you need to find this key '{0}'", strKey);
                                //strValue = strKey + "_value";
                                strValue = ResolveInternalKey(strKey, jobContext);
                            }
                            nLength = nStart - nCurrent;
                            if (nLength > 0)
                            {
                                //copy possible original part before token
                                fwrite(line + nCurrent, nLength, 1, sw);
                            }
                            // debugging
                            //Console.Write(line.Substring(nCurrent, nLength));
                            //ConfigLogger.Instance.LogDebug(line.Substring(nCurrent, nLength));
                            //FILE_LOG(logDEBUG)<<"[ReplaceTemplateFile:target]"<<line;
                            // replace token
                            fwrite(strValue.c_str(), strValue.size(), 1, sw);
                            //Console.Write(strValue);
                            //ConfigLogger.Instance.LogDebug(strValue);
                            FILE_LOG(logDEBUG)<<"[ReplaceTemplateFile:strKey]"<<strKey<<"[strValue]"<<strValue;
                            // reset to restart
                            nCurrent = nEnd;
                            nStart = nEnd;
                            nResult++;
                            nStart = findStartChar(line, nStart);
                        }
                        nLength = nStart - nCurrent;
                        if (nLength > 0)
                        {
                            fwrite(line + nCurrent, nLength, 1, sw);
                        }
                    }

                }
                catch (runtime_error e)
                {
                    // Let the user know what went wrong.
                    //Console.WriteLine("The file could not be read:");
                    //ConfigLogger.Instance.LogError("ReplaceTemplateFile:" + e.Message);
                    FILE_LOG(logERROR)<<"[ReplaceTemplateFile]"<<e.what();
                    //Console.WriteLine(e.Message);
                }
                fclose(sw);
            }
            else
            {
                FILE_LOG(logERROR)<<"[ReplaceTemplateFile] cannot open writeout file "<<strTargetName;
            }
            fclose(sr);
        }
        else
        {
            FILE_LOG(logERROR)<<"[ReplaceTemplateFile] cannot open readin file "<<strTemplateName;
        }
        return nResult;
    }

    int TemplateManager::GenerateScriptFile(string strPlatform, string strTemplateName, string strScriptName, StringMap& jobContext)
    {
        StringMap hashTable;
        if (XmlManager::GetHashTable(strPlatform, hashTable))
        {
            return ReplaceTemplateFile(strTemplateName, strScriptName, hashTable, jobContext);
        }
        else
        {
            return -1;
        }
    }
}
