#include "SubmitWidget.h"
#include <Wt/WIntValidator>
#include <Wt/WBreak>
#include <Wt/WPanel>
#include <Wt/WTable>
#include <boost/lexical_cast.hpp>

SubmitWidget::SubmitWidget(const std::string& strFolderName, WContainerWidget* parent)
    :WContainerWidget(parent), m_folderName(strFolderName), m_parent(parent)
        , m_notifyObj(NULL)
{


    ///////////////////////////////////
    fileTreeViewMsg = new WText("select rendering file by checkboxing");
    addWidget(fileTreeViewMsg);

    fileTreeView = new FileTreeView(strFolderName, this);

    WPanel* panel = new WPanel(this);
    panel->resize(600, 300);
    panel->setTitle("select rendering file:");
    panel->setCollapsible(true);

    panel->setCentralWidget(fileTreeView);

    addWidget(new WBreak());
    //////////////////////////////////
    WTable* layout =  new WTable(this);
    ////////////////////////////////////////
    layout->elementAt(1,0)->addWidget(new WText("select number of rendering nodes:"));
    layout->elementAt(1,0)->setContentAlignment(Wt::AlignRight);

    instanceNumberBox = new WSpinBox();
    instanceNumberBox->setRange(1, 10);
    instanceNumberBox->setValue(1);
    layout->elementAt(1,1)->addWidget(instanceNumberBox);
    //addWidget(new WBreak());
    //////////////////////////////////
    layout->elementAt(2,0)->addWidget(new WText("select output format:"));
    layout->elementAt(2,0)->setContentAlignment(Wt::AlignRight);
    outputFormatBox = new WComboBox();
    outputFormatBox->addItem("JPEG");
    outputFormatBox->addItem("PNG");
    outputFormatBox->addItem("BMP");
    layout->elementAt(2,1)->addWidget(outputFormatBox);
    //addWidget(new WBreak());
    /////////////////////////////////
    layout->elementAt(3, 0)->addWidget(new WText("select rendering node type:"));
    layout->elementAt(3,0)->setContentAlignment(Wt::AlignRight);
    instanceTypeBox = new DBEnumComboBox("EC2_InstanceTypes");
    layout->elementAt(3,1)->addWidget(instanceTypeBox);
    //    addWidget(new WBreak());
    ////////////////////////////////
    layout->elementAt(4, 0)->addWidget(new WText("select starting rendering frame:"));
    layout->elementAt(4,0)->setContentAlignment(Wt::AlignRight);
    startFrameBox = new WSpinBox();
    startFrameBox->setMinimum(1);
    startFrameBox->setValue(1);
    layout->elementAt(4,1)->addWidget(startFrameBox);
    //addWidget(new WBreak());
    ///////////////////////////////
    layout->elementAt(5,0)->addWidget(new WText("select ending rendering frame:"));
    layout->elementAt(5,0)->setContentAlignment(Wt::AlignRight);
    endFrameBox = new WSpinBox();
    endFrameBox->setMinimum(1);
    endFrameBox->setValue(1);
    layout->elementAt(5,1)->addWidget(endFrameBox);
    endFrameMsg = new WText("cannot be less than starting frame number!");
    endFrameMsg->decorationStyle().setForegroundColor(Wt::red);
    layout->elementAt(5,2)->addWidget(endFrameMsg);
    //addWidget(new WBreak());
    ///////////////////////////////////////////////////


    //addWidget(new WBreak());

    msg = new WText("debug msg:");
    msg->decorationStyle().setForegroundColor(Wt::blue);
    msg->decorationStyle().setTextDecoration(WCssDecorationStyle::Overline);
    layout->elementAt(6, 0)->addWidget(msg);
    layout->elementAt(6, 0)->setColumnSpan(2);
    layout->elementAt(6, 0)->setRowSpan(6);
    //////////////////////////////////
    submitButton = new WPushButton("submit", layout->elementAt(13,0));
    layout->elementAt(13,0)->setContentAlignment(Wt::AlignRight);

    submitButton->clicked().connect(this, &SubmitWidget::submit);
    summaryButton = new WPushButton("summary", layout->elementAt(13,1));
    layout->elementAt(13,1)->setContentAlignment(Wt::AlignLeft);
    summaryButton->clicked().connect(this, &SubmitWidget::summary);

    submitButton->setEnabled(false);

    //endFrameBox->valueChanged().connect(this, &SubmitWidget::validateFrameNumber);

}

void SubmitWidget::setupNotifyEventMethod(NotifyMethod notifyMethod, WObject* notifyObj)
{
    m_notifyMethod = notifyMethod;
    m_notifyObj = notifyObj;
}

void SubmitWidget::submit()
{
    bool bSubmitResult = true;
    std::string str;
    if (m_notifyObj)
    {
        boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(bSubmitResult ? SubmitSuccessful:SubmitFailed);
    }
    str = "submit ";
    str += bSubmitResult? "successful!" : "failed!";

    msg->setText(str);
    submitButton->setEnabled(false);
}

void SubmitWidget::summary()
{
    if (endFrameBox->value() < startFrameBox->value())
    {
        endFrameMsg->decorationStyle().setForegroundColor(Wt::red);
        endFrameMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::Blink);
        return;
    }
    else
    {
        endFrameMsg->decorationStyle().setForegroundColor(Wt::blue);
        endFrameMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::LineThrough);
    }

    std::string path;
    if (!fileTreeView->getCheckedFile(path))
    {
        fileTreeViewMsg->decorationStyle().setForegroundColor(Wt::red);
        fileTreeViewMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::Blink);
        return;
    }
    else
    {
        fileTreeViewMsg->decorationStyle().setForegroundColor(Wt::blue);
        fileTreeViewMsg->decorationStyle().setTextDecoration(WCssDecorationStyle::LineThrough);
    }


    std::string str;

    str += "<p>RenderFile:<b>" + path + "</b></p>";

    str += "<p>InstanceNumber:<b>" + boost::lexical_cast<std::string>(instanceNumberBox->value()) + "</b></p>";
    str += "<p>OutputFormat:<b>" + outputFormatBox->currentText().toUTF8() + "</b></p>";
    str += "<p>InstanceType:<b>" + instanceTypeBox->currentText().toUTF8() + "</b></p>";
    str += "<p>StartFrame:<b>" + boost::lexical_cast<std::string>(startFrameBox->value()) + "</b></p>";
    str += "<p>EndFrame:<b>" + boost::lexical_cast<std::string>(endFrameBox->value()) + "</b></p>";
    str += "To proceed, click submit button";
    msg->setText(str);
    submitButton->setEnabled(true);
}








